# Channel.pm

# ===========================================================================
# file         : Channel.pm
# copyright    : 2005, 2006 by Sven 'ScP' Paulsen & Felix 'WeeZer' Siefert
# email        : scp@planetteamspeak.com & weezer@planetteamspeak.com
# version      : 0.3.0
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.
# ===========================================================================

package TeamSpeak2::TCPquery::Channel;

use strict;
use warnings;
use diagnostics;

use Data::Dumper;

use constant PACKAGE => {
  "name"    => "TeamSpeak2::TCPquery::Channel",
  "version" => "0.3.0",
};

use constant SUCCESS => "OK";

# ===========================================================================
# There should be no need to change anything below
# ===========================================================================

# Class method: new()
# Creates a new TeamSpeak2::TCPquery::Channel
#
sub new {
  my ($class, $tcpquery, @channel) = @_;
  my $self = bless {
    tcpquery => $tcpquery,
    server   => $tcpquery->{selected},
    id       => $channel[0],
    codec    => $channel[1],
    parent   => $channel[2],
    order    => $channel[3],
    maxusers => $channel[4],
    name      => $channel[5],
    flags     => {
      total        => $channel[6],
      unregistered => ($channel[6] & 1),
      moderated    => ($channel[6] & 2),
      password     => ($channel[6] & 4),
      subchannel   => ($channel[6] & 8),
      default      => ($channel[6] & 16),
    },        
    password => $channel[7],
    topic    => $channel[8],
  };
  return $self;
}

# Object method: sendMessage()
# Sends one ore more messages to all players in the channel and returns TRUE
# or FALSE accordingly
#
sub sendMessage {
  my ($self, @messages) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $errors = 0;
  my @players = $self->getPlayers();
  foreach my $message (@messages) {
    foreach my $player (@players) {
      $errors = 1 if(!$player->sendMessage($message));
    }
  }
  return 0 if($errors);
  return 1;
}

# Object method: move()
# Moves all players in the channel into another given channel and returns
# TRUE or FALSE accordingly
#
sub move {
  my ($self, $channel_id) = @_;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->move($channel_id));
  }
  return 0 if($errors);
  return 1;
}

# Object method: remove()
# Closes the connection of all players in the channel and returns TRUE or
# FALSE accordingly
#
sub remove {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->remove());
  }
  return 0 if($errors);
  return 1;
}

# Object method: kick()
# Kicks all players in the channel from the server and returns TRUE or
# FALSE accordingly
#
sub kick {
  my ($self, $reason) = @_;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->kick($reason));
  }
  return 0 if($errors);
  return 1;
}

# Object method: ban()
# Bans all players in the channel from the server and returns TRUE or
# FALSE accordingly
#
sub ban {
  my ($self, $time, $reason) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->ban($time, $reason));
  }
  return 0 if($errors);
  return 1;
}

# Object method: banIp()
# Bans the Ip addresses of all players in the channel without kicking them
# from the server and returns TRUE or FALSE accordingly
#
sub banIP {
  my ($self, $time) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->banIp($time));
  }
  return 0 if($errors);
  return 1;
}

# Object method: setServeradmin()
# Changes ServerAdmin permissions of all players in th channel and returns
# TRUE or FALSE accordingly
#
sub setServeradmin {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->setServeradmin($value));
  }
  return 0 if($errors);
  return 1;
}

# Object method: setSticky()
# Sticks all player to the channel and returns TRUE or FALSE accordingly
#
sub setSticky {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->setSticky($value));
  }
  return 0 if($errors);
  return 1;
}

# Object method: removeRegistration()
# Deletes the registrations of all players in the channel and returns
# TRUE or FALSE accordingly
#
sub removeRegistration {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $errors = 0;
  foreach my $player ($self->getPlayers()) {
    $errors = 1 if(!$player->removeRegistration());
  }
  return 0 if($errors);
  return 1;
}

# Object method: getChannelinfo()
# Returns an array containing various information about the player
#
sub getChannelinfo {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  return $self->{tcpquery}->getChannelinfo($self->{id});
}

# Object method: getPlayers()
# Returns an array containing information of all players in the channel
#
sub getPlayers {
  my $self = shift;
  my @players = ();
  foreach my $player ($self->{tcpquery}->getLastPlayerlist) {
    if($player->{c_id} == $self->{id}) {
      push(@players, $player);
    }
  }
  return @players;
}

# Object method: getSubChannels()
# Returns an array containing information of all sub-channels of the channel
#
sub getSubChannels {
  my $self = shift;
  my @subchannels = ();
  return @subchannels if($self->{parent} != -1);
  foreach my $channel ($self->{tcpquery}->getLastChannellist) {
    if($channel->{parent} == $self->{id}) {
      push(@subchannels, $channel);
    }
  }
  return @subchannels;
}

# Object method: getFlags()
# Returns a string containing the channels current flags as the TeamSpeak 2
# client software would show them
#
sub getFlags {
  my $self = shift;
  my $flags = "(";
  if($self->{flags}->{unregistered}) {
    $flags .= "U";
  } else {
    $flags .= "R";
  }
  $flags .= "M" if($self->{flags}->{moderated});
  $flags .= "P" if($self->{flags}->{password});
  $flags .= "S" if($self->{flags}->{subchannel});
  $flags .= "D" if($self->{flags}->{default});
  $flags .= ")";
  return $flags;
}

# Object method: getCodec()
# Returns a string containing the channels current codec as the TeamSpeak 2
# client software would show it
#
sub getCodec {
  my $self = shift;
  return "CELP 5.2" if(!$self->{codec});
  return "CELP 6.3" if($self->{codec} == 1);
  return "GSM 14.8" if($self->{codec} == 2);
  return "GSM 16.4" if($self->{codec} == 3);
  return "Windows CELP 5.2" if($self->{codec} == 4);
  return "Speex 3.4" if($self->{codec} == 5);
  return "Speex 5.2" if($self->{codec} == 6);
  return "Speex 7.2" if($self->{codec} == 7);
  return "Speex 9.3" if($self->{codec} == 8);
  return "Speex 12.3" if($self->{codec} == 9);
  return "Speex 16.3" if($self->{codec} == 10);
  return "Speex 19.5" if($self->{codec} == 11);
  return "Speex 25.9" if($self->{codec} == 12);
  return "unknown";
}

1;