# Player.pm

# ===========================================================================
# file         : Player.pm
# copyright    : 2005, 2006 by Sven 'ScP' Paulsen & Felix 'WeeZer' Siefert
# email        : scp@planetteamspeak.com & weezer@planetteamspeak.com
# version      : 0.3.4
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 675 Mass Ave, Cambridge, MA 02139, USA.
# ===========================================================================

package TeamSpeak2::TCPquery::Player;

use strict;
use warnings;
use diagnostics;

use Socket;
use Data::Dumper;

use constant PACKAGE => {
  "name"    => "TeamSpeak2::TCPquery::Player",
  "version" => "0.3.4",
};

use constant SUCCESS => "OK";

# ===========================================================================
# There should be no need to change anything below
# ===========================================================================

# Class method: new()
# Creates a new TeamSpeak2::TCPquery::Player object
#
sub new {
  my ($class, $tcpquery, @player) = @_;
  my $self = bless {
    tcpquery  => $tcpquery,
    server    => $tcpquery->{selected},
    p_id      => $player[0],
    c_id      => $player[1],
    ps        => $player[2],
    bs        => $player[3],
    pr        => $player[4],
    br        => $player[5],
    pl        => $player[6],
    ping      => $player[7],
    logintime => $player[8],
    idletime  => $player[9],
    cprivs    => {
      total         => $player[10],
      channeladmin  => ($player[10] & 1),
      operator      => ($player[10] & 2),
      voice         => ($player[10] & 4),
      auto_operator => ($player[10] & 8),
      auto_voice    => ($player[10] & 16),
    },        
    pprivs    => {
      total       => $player[11],
      serveradmin => ($player[11] & 1),
      allowed_reg => ($player[11] & 2),
      registered  => ($player[11] & 4),
      sticky      => ($player[11] & 16),
    },
    pflags    => {
      total       => $player[12],
      c_commander => ($player[12] & 1),
      v_requested => ($player[12] & 2),
      no_whispers => ($player[12] & 4),
      away        => ($player[12] & 8),
      mic_muted   => ($player[12] & 16),
      snd_muted   => ($player[12] & 32),
      recording   => ($player[12] & 64),
    },    
    ip        => $player[13],
    nick      => $player[14],
    loginname => $player[15],
  };  
  return $self;
}

# Object method: sendMessage()
# Sends one ore more messages to the player and returns TRUE or FALSE
# accordingly
#
sub sendMessage {
  my ($self, @messages) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  foreach(@messages) {
    $self->{tcpquery}->_sendCommand("msgu $self->{p_id} @".$_);
    return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  }
  return 1;
}

# Object method: move()
# Moves the player into a given channel and returns TRUE or FALSE
# accordingly
#
sub move {
  my ($self, $channel_id) = @_;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->{tcpquery}->_sendCommand("mptc $channel_id $self->{p_id}");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: remove()
# Closes the players connection and returns TRUE or FALSE accordingly
#
sub remove {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  $self->{tcpquery}->_sendCommand("removeclient $self->{p_id}");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: kick()
# Kicks the player with or without a reason and returns TRUE or FALSE
# accordingly
#
sub kick {
  my ($self, $reason) = @_;
  $self->{tcpquery}->_checkLoginLvl("serveradmin",(caller(0))[3]);
  if(defined($reason)) {
    $self->{tcpquery}->_sendCommand("kick $self->{p_id} $reason");
  } else {
    $self->{tcpquery}->_sendCommand("kick $self->{p_id}");
  }
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: ban()
# Bans the player with or without a reason and returns TRUE or FALSE
# accordingly
#
sub ban {
  my ($self, $time, $reason) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  if(defined($time)) {
    $self->{tcpquery}->_sendCommand("banplayer $self->{p_id} $time");
  } else {
    $self->{tcpquery}->_sendCommand("banplayer $self->{p_id}");
  }
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS || $self->kick($reason));
  return 1;
}

# Object method: banIp()
# Bans the players IP address without kicking him from the server and
# returns TRUE or FALSE accordingly
#
sub banIp {
  my ($self, $time) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  if(defined($time)) {
    $self->{tcpquery}->_sendCommand("banadd $self->{ip} $time");
  } else {
    $self->{tcpquery}->_sendCommand("banadd $self->{ip}");
  }
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: setServeradmin()
# Grants or revokes a players ServerAdmin permissions and returns TRUE or
# FALSE accordingly
#
sub setServeradmin {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->{tcpquery}->_sendCommand("sppriv $self->{p_id} privilege_serveradmin $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: setSticky()
# Sticks a player to the channel he is in and returns TRUE or FALSE accordingly
#
sub setSticky {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->{tcpquery}->_sendCommand("sppriv $self->{p_id} privilege_channelsticky $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: removeRegistration()
# Deletes a players registration and returns TRUE or FALSE accordingly
#
sub removeRegistration {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $p_dbid = $self->getDatabaseId();
  $self->{tcpquery}->_sendCommand("dbuserdel $p_dbid");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: getPlayerinfo()
# Returns an array containing various information about the player
#
sub getPlayerinfo {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  return $self->{tcpquery}->getPlayerinfo($self->{p_id});
}

# Object method: getChannel()
# Returns various information about the channel, in which the player is
#
sub getChannel {
  my $self = shift;
  foreach my $channel ($self->{tcpquery}->getLastChannellist()) {
    return $channel if($channel->{id} == $self->{c_id});
  }
  return 0;
}

# Object method: getDatabaseId()
# Returns the players database ID as an integer
#
sub getDatabaseId {
  my $self = shift;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  $self->{tcpquery}->_sendCommand("dbuserid $self->{loginname}");
  foreach($self->{tcpquery}->_getMultipleLines()) {
    return $1 if(/^(\d+)\s*$/);
  }
  return 0;
}

# Object method: changePassword()
# Changes a users login password and returns TRUE or FALSE accordingly
#
sub changePassword {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $p_dbid = $self->getDatabaseId();
  $self->{tcpquery}->_sendCommand("dbuserchangepw $p_dbid $value $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: changeDbPermission()
# Changes a users serveradmin status in the database and returns TRUE or
# FALSE accordingly
#
sub changeDbPermission {
  my ($self, $value) = @_;
  $self->{tcpquery}->_checkLoginLvl("superadmin",(caller(0))[3]);
  my $p_dbid = $self->getDatabaseId();
  $self->{tcpquery}->_sendCommand("dbuserchangeattribs $p_dbid $value");
  return 0 if($self->{tcpquery}->_getSingleLine() ne SUCCESS);
  return 1;
}

# Object method: getFlags()
# Returns a string containing the players current flags as the TeamSpeak 2
# client software would show them
#
sub getFlags {
  my $self = shift;
  my $flags = "(";
  if($self->{pprivs}->{registered}) {
    $flags .= "R";
  } else {
    $flags .= "U";
  }
  $flags .= " St" if($self->{pprivs}->{sticky});
  $flags .= " SA" if($self->{pprivs}->{serveradmin});
  $flags .= " CA" if($self->{cprivs}->{channeladmin});
  $flags .= " AO" if($self->{cprivs}->{auto_operator});
  $flags .= " AV" if($self->{cprivs}->{auto_voice});
  $flags .= " O" if($self->{cprivs}->{operator});
  $flags .= " V" if($self->{cprivs}->{voice});
  $flags .= " Rec" if($self->{pflags}->{recording});
  $flags .= ")";
  $flags .= " WV:" if($self->{pflags}->{v_requested});
  return $flags;
}

# Object method: getPermissionRank()
# Returns the players permission rank
#
sub getFlagRanking {
  my $self = shift;
  return 1 if($self->{pprivs}->{serveradmin});
  return 2 if($self->{cprivs}->{channeladmin});
  return 3 if($self->{cprivs}->{operator});
  return 4 if($self->{cprivs}->{voice});
  return 5 if($self->{cprivs}->{auto_operator});
  return 6 if($self->{cprivs}->{auto_voice});
  return 7 if($self->{pprivs}->{registered});
  return 8;
}

# Object method: getHostname()
# Returns a string containing the players hostname by using Perl's Socket
# module
#
sub getHostname {
  my $self = shift;
  my $iaddr = inet_aton($self->{ip});
  if(my $hostname = gethostbyaddr($iaddr, AF_INET)) {
    return $hostname;  
  } else {
    return "unknown";
  }  
}

1;