# ts2perlmod-lib.pl
# This file contains common functions used by the TS2PerlMod script

use IO::File;
use Color::Output;
use Config::IniFiles;
use RTF::Tokenizer;

use POSIX qw(strftime);

use constant LOG => 0;

# Function: init()
# Performs various actions at startup
#
sub init {
  # init Color::Output package
  Color::Output::Init;
  # print copyright information
  &printColored(0, SCRIPT->{name}." [Version ".SCRIPT->{version}."]");
  &printColored(0, "Copyright (c) 2005 - 2007 by Sven 'ScP' Paulsen");
  &printColored(0, "");
  # get attached command-line arguments
  &getArguments();
}

# Function: getArguments()
# Reads attached command-line arguments and changes the scripts settings
#
sub getArguments() {
  foreach(@ARGV) {
    if(/^\-config=([^\r\n\f]+)\s*$/) {
      # check if we shall use another configuration file
      SCRIPT->{config} = $1;
    } elsif(/^\-pidfile=([^\r\n\f]+)\s*$/) {
      # check if we shall use another PID file
      SCRIPT->{pidfile} = $1;
    } elsif(/^-daemon\s*$/) {
      # check if we shall daemonize the script
      SCRIPT->{daemon} = 1;
    }    
  }
}

# Function: daemonize()
# Can be used to daemonize the process on Linux by using fork()
#
sub daemonize {
  if(SCRIPT->{daemon}) {
    exit(0) if(fork());
    open(STDIN, "</dev/null") || return 0;
    open(STDOUT, ">/dev/null") || return 0;
    open(STDERR, ">&STDOUT") || return 0;
    open(PID, ">".SCRIPT->{pidfile}) || return 0;
    print PID $$;
    close PID;
    chdir("/");
    umask(0);
    return $$;
  } else {
    return 0;
  }
}

# Function: readConfig()
# Reads the scripts configuration file and returns a hash that is tied to the
# Config::IniFiles object
#
sub readConfig {
  # check if file exists
  if(!-f "./configs/".SCRIPT->{config}) {
    &abort("Configuration file ".SCRIPT->{config}." is missing");
  }
  # check if file is readable
  if(!-r "./configs/".SCRIPT->{config}) {
    &abort("Configuration file ".SCRIPT->{config}." is not readable");
  }
  # create a tied hash containing the scripts settings
  tie(my %config, "Config::IniFiles", (-file => "./configs/".SCRIPT->{config}));
  &printColored("ok", "Using configuration file ".SCRIPT->{config});
  return (%config);
}

# Function: checkFeatureSettings()
# Reads the scripts settings and returns a hash containing all options
#
sub checkFeatureSettings {
  my ($ts2, $config) = @_;
  my %config = %$config;
  my %settings;
  &printColored(0, "");
  # check settings for the Extended Welcome Message feature
  if($config{"EXTENDED-WELCOME-MESSAGE"}{"Enabled"}) {
    my @content = &readFile("Welcome Message:", $config{"EXTENDED-WELCOME-MESSAGE"}{"File"});
    $settings{"ewm"}{"content"} = \@content;
  }
  # check settings for the Bad Nickname feature
  if($config{"BAD-NICKNAME"}{"Enabled"}) {
    my @content = &readFile("Bad Nickname:   ",$config{"BAD-NICKNAME"}{"File"});
    $settings{"badnick"}{"content"} = \@content;
    $settings{"badnick"}{"punishment"} = $config{"BAD-NICKNAME"}{"Punishment"};
    $settings{"badnick"}{"reason"} = $config{"BAD-NICKNAME"}{"Reason"};
    $settings{"badnick"}{"message"} = $config{"BAD-NICKNAME"}{"Message"};
    if($settings{"badnick"}{"punishment"} eq "move") {
      $settings{"badnick"}{"channel"} = &searchChannel("Bad Nickname:   ",$ts2,$config{"BAD-NICKNAME"}{"Channel"});
    }
  }
  # check settings for the Bad Channel feature
  if($config{"BAD-CHANNEL"}{"Enabled"}) {
    my @content = &readFile("Bad Channel:    ",$config{"BAD-CHANNEL"}{"File"});
    $settings{"badchan"}{"content"} = \@content;
    $settings{"badchan"}{"punishment"} = $config{"BAD-CHANNEL"}{"Punishment"};
    $settings{"badchan"}{"reason"} = $config{"BAD-CHANNEL"}{"Reason"};
    $settings{"badchan"}{"message"} = $config{"BAD-CHANNEL"}{"Message"};
    if($settings{"badchan"}{"punishment"} eq "move") {
      $settings{"badchan"}{"channel"} = &searchChannel("Bad Channel:    ",$ts2,$config{"BAD-CHANNEL"}{"Channel"});
    }
  }
  # check settings for the Move Idlers feature
  if($config{"MOVE-IDLERS"}{"Enabled"}) {
    $settings{"moveidler"}{"channel"} = &searchChannel("Move Idlers:    ",$ts2,$config{"MOVE-IDLERS"}{"Channel"});
    $settings{"moveidler"}{"idletime"} = $config{"MOVE-IDLERS"}{"Idletime"} * 60;
  }
  # check settings for the Warn Idlers feature
  if($config{"WARN-IDLERS"}{"Enabled"}) {
    $settings{"warnidler"}{"message"} = $config{"WARN-IDLERS"}{"Message"};
    $settings{"warnidler"}{"idletime"} = $config{"WARN-IDLERS"}{"Idletime"} * 60;  
  }
  # check settings for the Kick Idlers feature
  if($config{"KICK-IDLERS"}{"Enabled"}) {    
    my $targets = 0;
    $targets = $targets+2 if($config{"KICK-IDLERS"}{"TargetPlayerR"});
    $targets = $targets+1 if($config{"KICK-IDLERS"}{"TargetPlayerU"});
    $targets = $targets+8 if($config{"KICK-IDLERS"}{"TargetChannelR"});
    $targets = $targets+4 if($config{"KICK-IDLERS"}{"TargetChannelU"});
    $settings{"kickidler"}{"targets"} = $targets;
    $settings{"kickidler"}{"reason"} = $config{"KICK-IDLERS"}{"Reason"};
    $settings{"kickidler"}{"idletime"} = $config{"KICK-IDLERS"}{"Idletime"};
  }
  # check settings for the Anti Recording feature
  if($config{"ANTI-REC"}{"Enabled"}) {
    $settings{"antirec"}{"punishment"} = $config{"ANTI-REC"}{"Punishment"};
    $settings{"antirec"}{"reason"} = $config{"ANTI-REC"}{"Reason"};
    $settings{"antirec"}{"message"} = $config{"ANTI-REC"}{"Message"};
    if($settings{"antirec"}{"punishment"} eq "move") {
      $settings{"antirec"}{"channel"} = &searchChannel("Anti Recording: ",$ts2,$config{"ANTI-REC"}{"Channel"});
    }
  }
  # check settings for Anti Flood feature
  if($config{"ANTI-FLOOD"}{"Enabled"}) {
    $settings{"antiflood"}{"connections"} = $config{"ANTI-FLOOD"}{"MaxConnections"};
    $settings{"antiflood"}{"seconds"} = $config{"ANTI-FLOOD"}{"MaxSeconds"};
  }
  # check settings for the Take Over protection feature
  if($config{"TAKE-OVER-PROTECTION"}{"Enabled"}) {
    $settings{"takeover"}{"punishment"} = $config{"TAKE-OVER-PROTECTION"}{"Punishment"};
    $settings{"takeover"}{"reason"} = $config{"TAKE-OVER-PROTECTION"}{"Reason"};
    $settings{"takeover"}{"message"} = $config{"TAKE-OVER-PROTECTION"}{"Message"};
    if($settings{"takeover"}{"punishment"} eq "move") {
      $settings{"takeover"}{"channel"} = &searchChannel("TO Protection: ",$ts2,$config{"TAKE-OVER-PROTECTION"}{"Channel"});
    }
  }
  # check settings for the Protected Users feature
  if($config{"PROTECTED-USERS"}{"Enabled"}) {
    my @content = &readFile("Protected Users:",$config{"PROTECTED-USERS"}{"File"});
    $settings{"protectuser"}{"content"} = \@content;
  }
  # check settings for the Auto AFK feature
  if($config{"AUTO-AFK"}{"Enabled"}) {
    $settings{"autoafk"}{"channel"} = &searchChannel("Auto AFK:       ",$ts2,$config{"AUTO-AFK"}{"Channel"});
  }
  # check settings for the Advert Message feature
  if($config{"ADVERT-MESSAGE"}{"Enabled"}) {
    my @content = &readFile("Advert Message: ",$config{"ADVERT-MESSAGE"}{"File"});
    $settings{"advert"}{"content"} = \@content;
    $settings{"advert"}{"interval"} = $config{"ADVERT-MESSAGE"}{"Interval"};
  }
  # check settings for the Stats Message feature
  if($config{"STATS-MESSAGE"}{"Enabled"}) {
    my @content = &readFile("Stats Message:  ",$config{"STATS-MESSAGE"}{"File"});
    $settings{"stats"}{"content"} = \@content;
    $settings{"stats"}{"interval"} = $config{"STATS-MESSAGE"}{"Interval"};
  }
  # check settings for the Sticky Channel feature
  if($config{"STICKY-CHANNEL"}{"Enabled"}) {
    $settings{"sticky"}{"channel"} = &searchChannel("Sticky Channel: ",$ts2,$config{"STICKY-CHANNEL"}{"Channel"});
  }
  # check settings for the Update Banlist feature
  if($config{"UPDATE-BANLIST"}{"Enabled"}) {
    $settings{"banupd"}{"mins"} = $config{"UPDATE-BANLIST"}{"Bantime"};
  }
  # check settings for the Auto Servertype feature
  if($config{"AUTO-SERVERTYPE"}{"Enabled"}) {
    $settings{"serverset"}{"clients"} = $config{"AUTO-SERVERTYPE"}{"Clients"};
    $settings{"serverset"}{"interval"} = $config{"AUTO-SERVERTYPE"}{"Interval"};
  }
  # check Repeat Timer settings
  if($config{"TIMER"}{"RepeatTimer"} < 2) {
    &printColored(0, "");
    &printColored("**", "Repeat Timer setting is invalid and has been corrected automatically");
    $settings{"timer"} = 2;
  } else {
    $settings{"timer"} = $config{"TIMER"}{"RepeatTimer"};
  }
  &printColored(0, "");
  &printColored("ok", "Initialization sequence completed");
  &printColored("ok", SCRIPT->{name}." started with PID ".$$);
  return \%settings;
}

# Function: readFile()
# Opens the given textfile and returns an array containing the files content
#
sub readFile {
  my ($feature, $file) = @_;
  # check if file is exists
  if(!-f "./text/".$file) {
    &abort($feature."  File ".$file." is missing");
  }
  # check if file is readable
  if(!-r "./text/".$file) {
    &abort($feature."  File ".$file." is not readable");
  }
  # create a new filehandle using IO::File
  my $fh = new IO::File;
  # open the file
  $fh->open("< ./text/".$file);
  # create an array containing the files content
  my @content = split("\n", <$fh>);
  &printColored("ok", $feature."  Imported content from file ".$file);
  return @content;
}

# Function: openLog()
# Opens a new logfile to store various information
#
sub openLog {
  my $file = shift;
  # check if file is exists
  # create a new filehandle using IO::File
  my $fh = new IO::File;
  $fh->autoflush(1);
  # open the file
  if(!$fh->open(">> ./logs/".$file)) {
    &abort("File ".$file." cannot be used as logfile");
  }
  &printColored("ok", "Using logfile ".$file);
  return $fh;
}

# Function: log()
# Adds a new line to the scripts logfile
#
sub log {
  my ($log, $message) = @_;
  my $current_time = strftime "%m/%d/%y %H:%M:%S", localtime;
  print $log $current_time."\t".$message."\n" if($log);
}

# Function: searchChannel()
# Searches of a channel based on a string and returns an array containing a
# Teamspeak2::TCPquery::Channel object
#
sub searchChannel {
  my ($feature, $tcpquery, $string) = @_;
  # get channellist from server
  foreach($tcpquery->getLastChannellist()) {
    # check if the channel is registered and if its name contains the needed string
    my $pattern = lc($string);
    if($pattern !~ /^[\w\s]+$/) {
      &abort($feature."  Channel setting contains invalid characters (".$string.")");
    }
    if(lc($_->{name}) =~ /$pattern/ && !$_->{flags}->{unregistered}) {
      &printColored("ok", $feature."  Using channel with ID ".$_->{id}." (".$_->{name}.")");
      # return the channel object
      return $_;
    }
  }
  # abort if no channel was found
  &abort($feature."  Could not find a registered channel containing the configured string (".$string.")");
}

# Function: checkProtection()
# Checks if players account is on the list of protected users
#
sub checkProtection {
  my ($loginname, @protected) = @_;
  foreach(@protected) {
    return 1 if($_ eq $loginname);
  }
  return 0;
}

# Function: parseMessage()
# Parses each element of an array and replaces certain variables
#
sub parseMessage {
  my ($player, $channel, $server, $version, @message) = @_;
  my %replace = (
    # player variables
    "<PLAYER_ID>" => $player->{p_id},
    "<PLAYER_NICKNAME>" => $player->{nick},
    "<PLAYER_LOGINNAME>" => $player->{loginname},
    "<PLAYER_PACKETS_SENT>" => &addDots($player->{pr}),
    "<PLAYER_PACKETS_RECEIVED>" => &addDots($player->{ps}),
    "<PLAYER_DATA_SENT>" => &convertBytes($player->{br}),
    "<PLAYER_DATA_RECEIVED>" => &convertBytes($player->{bs}),
    "<PLAYER_PACKET_LOSS>" => &convertToPercent($player->{pl}),
    "<PLAYER_PING>" => $player->{ping},
    "<PLAYER_IP>" => $player->{ip},
    "<PLAYER_LOGINTIME>" => &formatTime($player->{logintime}),
    "<PLAYER_IDLETIME>" => &formatTime($player->{idletime}),
    "<PLAYER_FLAGS>" => $player->getFlags(),
    "<PLAYER_HOSTNAME>" => $player->getHostname(),
    # channel variables
    "<CHANNEL_ID>" => $channel->{id},
    "<CHANNEL_NAME>" => $channel->{name},
    "<CHANNEL_CODEC>" => $channel->getCodec(),
    "<CHANNEL_FLAGS>" => $channel->getFlags(),
    "<CHANNEL_ORDER>" => $channel->{order},
    "<CHANNEL_MAX_USERS>" => $channel->{maxusers},
    "<CHANNEL_TOPIC>" => $channel->{topic},
    # server variables
    "<SERVER_ID>" => $server->{server_id},
    "<SERVER_NAME>" => $server->{server_name},
    "<SERVER_PORT>" => $server->{server_udpport},
    "<SERVER_UPTIME>" => &formatTime($server->{server_uptime}),
    "<SERVER_VERSION>" => $version->{version},
    "<SERVER_LICENSE>" => $version->{license},
    "<SERVER_PLATFORM>" => $server->{server_platform},
    "<SERVER_MAX_USERS>" => $server->{server_maxusers},
    "<SERVER_CURRENT_USERS>" => $server->{server_currentusers},
    "<SERVER_CURRENT_CHANNELS>" => $server->{server_currentchannels},
    "<SERVER_PACKETS_SENT>" => &addDots($server->{server_packetssend}),
    "<SERVER_PACKETS_RECEIVED>" => &addDots($server->{server_packetsreceived}),
    "<SERVER_DATA_SENT>" => &convertBytes($server->{server_bytessend}),
    "<SERVER_DATA_RECEIVED>" => &convertBytes($server->{server_bytesreceived}),
    "<SERVER_BANDWIDTH_IN>" => &convertBytes($server->{server_bwinlastsec})."/s",
    "<SERVER_BANDWIDTH_OUT>" => &convertBytes($server->{server_bwoutlastsec})."/s",
    "<SERVER_BANDWIDTH_TOTAL>" => &convertBytes($server->{server_bwinlastsec} + $server->{server_bwoutlastsec})."/s",
    "<SERVER_WEBPOST_POSTURL>" => $server->{server_webpost_posturl},
    "<SERVER_WEBPOST_LINKURL>" => $server->{server_webpost_linkurl},
  );
  foreach my $line (@message) {
    for(keys %replace) {
      $line =~ s/$_/$replace{$_}/g;
    } 
  }
  return @message;
}

# Function: parseRtfString()
# Parses a given string for bad RTF codes
#
sub parseRtfString {
  my $string = shift;
  my $tokenizer = RTF::Tokenizer->new();
  $tokenizer->read_string($string);
  my ($token_type, $argument, $parameter) = $tokenizer->get_token();
  if($token_type eq "control" || $token_type eq "group") {
    if($string =~ /\\rtf/ || $string =~ /\\obj/) {
      return 0;
    }
  }
  return 1;
}

# Function: parseNullbyteString()
# Parses a given string for nullbyte chars
#
sub parseNullbyteString {
  my $string = shift;
  for(my $i = 0; $i < length($string); $i++) {
    return 0 if(ord(substr($string, $i, 1)) == 0);
  }
  return 1;
}

# Function: punish()
# The scripts settings and punishs a player accordingly
#
sub punish {
  my ($player, $settings) = @_;
  if($settings->{punishment} eq "move") {
    if($player->{c_id} != $settings->{channel}->{id} && !$player->{pprivs}->{sticky}) {
      if($player->move($settings->{channel}->{id})) {
        $player->setSticky(1) if(!$player->{pprivs}->{serveradmin});
        $player->sendMessage($settings->{message});
      } else {
        return 0;
      }
    }
  } elsif($settings->{punishment} eq "kick") {
    $player->kick($settings->{reason});
  } elsif($settings->{punishment} eq "remove") {
    $player->sendMessage($settings->{message});
    $player->remove();
  } elsif($settings->{punishment} eq "ban") {
    $player->ban(30, $settings->{reason});
  }
  return 0;
}

# Function: printColored()
# Prints a colored message to the console by using the Color::Output package
#
sub printColored {
  my ($type, @messages) = @_;
  foreach(@messages) {
    if($type) {
      if($type eq "ok") {
        cprintf("\x033%s\x030\x037%s\x030\x033%s\x030", "[", "ok", "]");
      } elsif($type eq "**") {
        cprintf("\x033%s\x030\x0313%s\x030\x033%s\x030", "[", "**", "]");
      } elsif($type eq "!!") {
        cprintf("\x033%s\x030\x035%s\x030\x033%s\x030", "[", "!!", "]");
      }
      cprintf("\x030%s\x030", " ");
    }
    cprintf("\x030%s\x030\n", $_);
  }
}

# Function: formatTime()
# Formats x seconds to a string like hD hh:mm:ss
#
sub formatTime {
	my $time = shift;
	my $seconds = $time % 60;
	my $minutes = ($time/60) % 60;
	my $hours = ($time/60/60) % 24;
	my $days = int($time/60/60/24);
	$seconds = "0".$seconds if($seconds < 10);
	$minutes = "0".$minutes if($minutes < 10);
	$hours = "0".$hours if($hours < 10);
	return $days."D ".$hours.":".$minutes.":".$seconds;
}

# Function: convertBytes()
# Converts bytes to kbytes, mbytes, gbytes, ...
#
sub convertBytes {
  my $bytes = shift;
  my $kbytes = sprintf('%.02f',$bytes/1024);
  my $mbytes = sprintf('%.02f',$kbytes/1024);
  my $gbytes = sprintf('%.02f',$mbytes/1024);
  if($gbytes >= 1) { return $gbytes." GByte"; }
  elsif($mbytes >= 1) { return $mbytes." MByte"; }
  elsif($kbytes >= 1) { return $kbytes." KByte"; }
  else { return $bytes." Byte"; }
}

# Function: addDots()
# Adds dots to a given integer
#
sub addDots {
  my $number = shift;
  1 while $number =~ s/^(-?\d+)(\d{3})/$1.$2/;
  return $number;
}

# Function: convertToPercent()
# Converts a given pl integer to a percent string
#
sub convertToPercent {
  my $number = shift;
  return ($number/100)."%";
}

# Function: abort()
# Stops the script and prints a colored error message
#
sub abort {
  my $error = shift;
  &printColored("!!", $error);
  if($^O eq "MSWin32") {
    cprintf("\x030\n%s\x030", "Press return key to exit...");
    system("pause > nul");
  }
  exit(1);
}

# Function: trim()
# Removes whitespaces from the start and end of a given string
#
sub trim {
  my $string = shift;
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;
  return $string;
}

1;