#!/usr/bin/perl

# ts2perlmod.pl - TS2PerlMod GNU/GPL-Edition
# This script runs in an infinite loop and performs various actions on your TeamSpeak server
# Written 2006 by Sven Paulsen

use strict;
use warnings;
use diagnostics;

use lib "packages";

require "./packages/lib-ts2perlmod.pl";

use Data::Dumper;
use TeamSpeak2::TCPquery;

use constant SCRIPT => {
  name    => "TS2PerlMod GNU/GPL-Edition",
  version => "0.9.19",
  config  => "default.ini",
  pidfile => "ts2perlmod.pid",
  daemon  => 0,
};

&init();

# read the scripts configuration file
my %config = &readConfig();

# open the scripts logfile
my $log;
$log = &openLog($config{"LOG"}{"File"}) if($config{"LOG"}{"Enabled"});
&log($log, SCRIPT->{name}." started");

# connect to server and create a new TeamSpeak2::TCPquery object
my $ts2 = TeamSpeak2::TCPquery->new(&trim($config{CONNECTION}{ServerAddress}),&trim($config{CONNECTION}{ServerTCPPort})) || &abort("Could not establish connection to server ".$config{CONNECTION}{ServerAddress});
&printColored("ok", "Connected to server ".$config{CONNECTION}{ServerAddress});

# disable debugging
$ts2->toggleDebug();

# check serverversion
my $version = $ts2->getServerversion();
if($version->{release} < 21) {
  &printColored("**", "The server is running an outdated version (".$version->{version}.")");
}

# select virtual server
$ts2->selectServer(&trim($config{CONNECTION}{ServerUDPPort})) || &abort("Could not select virtual server on port ".$config{CONNECTION}{ServerUDPPort});
&printColored("ok", "Selected virtual server on port ".$config{CONNECTION}{ServerUDPPort});

# authenticate as superadmin
$ts2->loginSuperadmin(&trim($config{LOGIN}{Username}),&trim($config{LOGIN}{Password})) || &abort("Could not authenticate as user ".$config{LOGIN}{Username});
&printColored("ok", "Authenticated as user ".$config{LOGIN}{Username});

# check the scripts settings
my %settings = %{&checkFeatureSettings($ts2,\%config)};
&log($log, "Initialization sequence completed");

&daemonize();

# declare some vars, arrays and hashes to store various information
my $last_stats = time();
my $sent_stats = 0;
my $last_advert = time();
my $last_serverset = time();
my %new_players;
my %nicks_not_checked;
my %exploits_not_checked;
my %set_sticky;
my %old_channels;
my %connections;

# create TeamSpeak2::TCPquery::Server object
my $server = $ts2->getServer($config{CONNECTION}{ServerUDPPort});

eval {

  # enter the main loop
  while(1) {
  
    # get the servers playerlist
    my @playerlist = $ts2->getPlayerlist();
  
    # get the servers channellist
    my @channellist = $ts2->getChannellist();
  
    # update TeamSpeak2::TCPquery::Server object
    $server->getServerinfo();
  
    # get the servers playerlist
    foreach my $player (@playerlist) {
    
      # create a new TeamSpeak2::TCPquery::Channel object
      my $channel = $player->getChannel();
      next if(!$channel);
    
      # check if current player is protected
      my $protected = &checkProtection($player->{loginname},@{$settings{protectuser}{content}}) if($config{"PROTECTED-USERS"}{"Enabled"});
    
      # extended welcome message
      if($config{"EXTENDED-WELCOME-MESSAGE"}{"Enabled"}) {
        if($player->{logintime} <= $settings{timer} && !exists($new_players{$player->{p_id}})) {
          $new_players{$player->{p_id}} = time();
          my @message = &parseMessage($player,$channel,$server,$version,@{$settings{ewm}{content}});
          $player->sendMessage(@message);
          &log($log, "[EXTENDED-WELCOME-MESSAGE] Sent welcome message to player ".$player->{nick}." ".$player->getFlags()." with IP ".$player->{ip});
        }
      }
    
      # sticky channel
      if($config{"STICKY-CHANNEL"}{"Enabled"}) {
        if(exists($set_sticky{$player->{p_id}}) && $player->{c_id} != $settings{sticky}{channel}->{id}) {
          delete $set_sticky{$player->{p_id}};
          $player->setSticky(0);
          &log($log, "[STICKY-CHANNEL] Released player ".$player->{nick}." ".$player->getFlags()." from channel ".$settings{sticky}{channel}->{name});
        }
        if($player->{c_id} == $settings{sticky}{channel}->{id} && !$player->{pprivs}->{sticky} && !$player->{pprivs}->{serveradmin}) {
          $set_sticky{$player->{p_id}} = 1 if(!exists($set_sticky{$player->{p_id}}));
          $player->setSticky(1);
          &log($log, "[STICKY-CHANNEL] Stuck player ".$player->{nick}." ".$player->getFlags()." to channel ".$settings{sticky}{channel}->{name});
        }
      }
    
      # move idlers
      if($config{"MOVE-IDLERS"}{"Enabled"} && !$protected) {
        if($player->{idletime} >= $settings{moveidler}{idletime} && !$player->{pprivs}->{sticky} && $player->{c_id} != $settings{moveidler}{channel}->{id}) {
          $player->move($settings{moveidler}{channel}->{id});
          &log($log, "[MOVE-IDLERS] Moved player ".$player->{nick}." ".$player->getFlags()." after ".formatTime($player->{idletime}));
        }
      }
    
      # warn idlers
      if($config{"WARN-IDLERS"}{"Enabled"} && !$protected) {
        if($player->{idletime} >= $settings{warnidler}{idletime} && $player->{idletime} <= ($settings{warnidler}{idletime} + $settings{timer})) {
          $player->sendMessage($settings{warnidler}{message});
          &log($log, "[WARN-IDLERS] Warned player ".$player->{nick}." ".$player->getFlags()." after ".formatTime($player->{idletime}));
        }
      }
    
      # bad nickname
      if($config{"BAD-NICKNAME"}{"Enabled"} && !$protected) {
        if($player->{logintime} <= $settings{timer} && !exists($nicks_not_checked{$player->{p_id}})) {
          $nicks_not_checked{$player->{p_id}} = time();
        }
        if(exists($nicks_not_checked{$player->{p_id}})) {
          foreach(@{$settings{badnick}{content}}) {
            if(lc($player->{nick}) =~ /$_/i) {
              &punish($player,$settings{badnick});
              &log($log, "[BAD-NICKNAME] Punished player ".$player->{nick}." ".$player->getFlags());
            }
          }
          delete $nicks_not_checked{$player->{p_id}};
        }
      }
      
      # bad channel
      if($config{"BAD-CHANNEL"}{"Enabled"} && !$protected) {
        if($channel->{flags}->{unregistered}) {
          foreach(@{$settings{badchan}{content}}) {
            if(lc($channel->{name}) =~ /$_/i) {
              &punish($player,$settings{badchan});
              &log($log, "[BAD-CHANNEL] Punished player ".$player->{nick}." ".$player->getFlags()." for beeing in channel ".$channel->{name});
            }
          }
        }
      }

      # anti rec
      if($config{"ANTI-REC"}{"Enabled"} && !$protected) {
        if($player->{pflags}->{recording}) {
          &punish($player,$settings{antirec});
          &log($log, "[ANTI-REC] Punished player ".$player->{nick}." ".$player->getFlags());
        }
      }
    
      # take over protection
      if($config{"TAKE-OVER-PROTECTION"}{"Enabled"} && !$protected) {
        if($player->{pprivs}->{serveradmin} && !$player->{pprivs}->{registered}) {
          $player->setServeradmin(0);
          &punish($player,$settings{takeover});
          &log($log, "[TAKE-OVER-PROTECTION] Punished player ".$player->{nick}." ".$player->getFlags());
        }
      }
    
      # auto afk
      if($config{"AUTO-AFK"}{"Enabled"} && !$protected) {
        if($player->{c_id} != $settings{autoafk}{channel}->{id} && $player->{pflags}->{away} && !$player->{pprivs}->{sticky}) {
          if($player->move($settings{autoafk}{channel}->{id})) {
            $old_channels{$player->{p_id}} = $player->{c_id} if(!exists($old_channels{$player->{p_id}}));
            &log($log, "[AUTO-AFK] Moved player ".$player->{nick}." into channel ".$settings{autoafk}{channel}->{name});
          }
        } elsif(!$player->{pflags}->{away} && exists($old_channels{$player->{p_id}})) {
          if($player->{c_id} == $settings{autoafk}{channel}->{id}) {
            $player->move($old_channels{$player->{p_id}});
          }
          &log($log, "[AUTO-AFK] Player ".$player->{nick}." ".$player->getFlags()." is no longer away");
          delete $old_channels{$player->{p_id}};
        }
      }
    
      # anti flood
      if($config{"ANTI-FLOOD"}{"Enabled"} && !$protected) {
        if($player->{logintime} <= $settings{timer}) {
          if(!exists($connections{$player->{ip}})) {
            $connections{$player->{ip}} = [time(), 1];
          } else {
            my $remaining_connections = $settings{antiflood}{connections}-@{$connections{$player->{ip}}}[1];
            @{$connections{$player->{ip}}}[1]++ if(time()-@{$connections{$player->{ip}}}[0] <= $settings{antiflood}{seconds});
            if(!$remaining_connections) {
              &log($log, "[ANTI-FLOOD] Added IP address ".$player->{ip}." to banlist");
              $player->banIp(5);
            }
          }
        }
      }
    
      # stats message
      if($config{"STATS-MESSAGE"}{"Enabled"}) {
        if(($last_stats + $settings{stats}{interval} * 60) < time()) {
          my @message = &parseMessage($player,$channel,$server,$version,@{$settings{stats}{content}});
          $player->sendMessage(@message);
          &log($log, "[STATS-MESSAGE] Sent stats message to player ".$player->{nick}." ".$player->getFlags());
          $sent_stats = 1;
        }
      }
      
      # bug detection
      if($config{"BUG-DETECTION"}{"Enabled"} && !$protected) {
        if($player->{logintime} <= $settings{timer} && !exists($exploits_not_checked{$player->{p_id}})) {
          $exploits_not_checked{$player->{p_id}} = time();
        }
        if(exists($exploits_not_checked{$player->{p_id}})) {
          if(length($player->{nick}) > 29 || !parseRtfString($player->{nick}) || !parseNullbyteString($player->{nick})) {
            $player->kick("exploit attempt detected");
            &log($log, "[BUG-DETECTION] Exploit attempt detected while checking nickname of player ".$player->{nick}." ".$player->getFlags());
          }
          delete $exploits_not_checked{$player->{p_id}};
        }
        if(length($channel->{name}) > 29 || !parseRtfString($channel->{name}) || !parseNullbyteString($channel->{name})) {
          $player->kick("exploit attempt detected");
          &log($log, "[BUG-DETECTION] Exploit attempt detected while checking channel of player ".$player->{nick}." ".$player->getFlags());
        }
      }
    
    }
  
    # kick idlers
    if($config{"KICK-IDLERS"}{"Enabled"}) {
      my $kicked = $server->kickIdlers($settings{kickidler}{idletime},$settings{kickidler}{targets},$settings{kickidler}{reason});
      my $x = "player";
      $x = "players" if($kicked > 1);
      &log($log, "[KICK-IDLERS] Kicked ".$kicked." idle ".$x." from server") if($kicked);
    }
    
    # update banlist
    if($config{"UPDATE-BANLIST"}{"Enabled"}) {
      foreach($ts2->getBanlist) {
        if($_->{mins} == "0") {
          $ts2->deleteBan($_->{b_id});
          $ts2->addBan($_->{ip}, $settings{banupd}{mins});
          &log($log, "[UPDATE-BANLIST] Converted permanent ban of IP ".$_->{ip}." into a ".$settings{banupd}{mins}." minute timeban");
        }
      }
    }
  
    # advert message
    if($config{"ADVERT-MESSAGE"}{"Enabled"}) {
      if(($last_advert + $settings{advert}{interval} * 60) < time()) {
        $server->sendMessage(@{$settings{advert}{content}});
        &log($log, "[ADVERT-MESSAGE] Sent advert message to all players");
        $last_advert = time();
      }
    }
    
    # auto servertype
    if($config{"AUTO-SERVERTYPE"}{"Enabled"}) {
      if(($last_serverset + $settings{serverset}{interval} * 60) < time()) {
        if($server->{server_currentusers} > $settings{serverset}{clients}) {
          if($server->{server_clan_server}) {
            $server->setModePublic();
            &log($log, "[AUTO-SERVERTYPE] Changed type of server ".$server->{server_name}." (".$server->{server_currentusers}."/".$server->{server_maxusers}.") to Public");
          }
        } else {
          if(!$server->{server_clan_server}) {
            $server->setModeClan();
            &log($log, "[AUTO-SERVERTYPE] Changed type of server ".$server->{server_name}." (".$server->{server_currentusers}."/".$server->{server_maxusers}.") to Clan");
          }
        }
        $last_serverset = time();
      }
    }
  
    # reset stats timer
    if($sent_stats) {
      $last_stats = time();
      $sent_stats = 0;
    }
  
    # delete outdated entries from hashes
    my $first_player = (@playerlist) ? shift(@playerlist) : 0;
    foreach(keys(%new_players)) { delete $new_players{$_} if((time()-$settings{timer}) > $new_players{$_}); }
    foreach(keys(%set_sticky)) { delete $set_sticky{$_} if($first_player && $_ < $first_player->{p_id}); }
    foreach(keys(%old_channels)) { delete $old_channels{$_} if($first_player && $_ < $first_player->{p_id}); }
    foreach(keys(%connections)) { delete $connections{$_} if(time()-@{$connections{$_}}[0] > $settings{antiflood}{seconds}); }
  
    # wait for x seconds
    sleep $settings{timer};
  
  }

};

# catch and log critical errors
if($@) {
  &log($log, &trim($@));
}

exit(1);